package pltag.german;

import java.util.ArrayList;


public class ElementaryTreeNode {

		private String phrasalCategory;
		private String functionalCategory;
		
		private ArrayList<ElementaryTreeNode> children = new ArrayList<ElementaryTreeNode>();
		private ElementaryTreeNode parent;
		private ElementaryTreeNode headChild;
		private boolean head;
		
		boolean isSubst = false;
		
		private MorphInfo morphology = new MorphInfo();
		
		boolean isLeaf = false;
		private String leaf = "";
		private String lemma = "";
		
		private boolean isTrace = false;
		private boolean isFiller = false;
		private String traceID;
		
		private TreeNode treeNodeLinkTop;
		private TreeNode treeNodeLinkBottom;
		private int nodeID;
		
		private ElementaryTree originTop;
		private ElementaryTree originBottom;
		
		public ElementaryTreeNode(String pCat, String fCat, TreeNode tnTop, TreeNode tnBot, int id) {
			phrasalCategory = pCat;
			functionalCategory = fCat;
			setTreeNodeTop(tnTop);
			setTreeNodeBottom(tnBot);
			nodeID = id;
		}
		
		public ElementaryTreeNode(String pCat, String fCat, TreeNode tnTop, TreeNode tnBot, int id, String l) {
			this(pCat, fCat, tnTop, tnBot, id);
			leaf = l;
			isLeaf = true;
		}
		
		// Basically a "copy" of the node, but without copying parent and children.
		public ElementaryTreeNode(ElementaryTreeNode node) {
			this(node.getPhrasalCategory(), node.getFunctionalCategory(), 
						node.getTreeNodeTop(), node.getTreeNodeBottom(), node.getID());
			if (node.isSubst()) {
				isSubst = true;
			}
			leaf = node.getLeaf();
			lemma = node.getLemma();
			if (node.isLeaf()) {
				isLeaf = true;
			}
			if (node.isTrace()) {
				isTrace = true;
			}
			if (node.isFiller()) {
				isFiller = true;
			}
			if (node.getTraceID() != null) {
				traceID = node.getTraceID();
			}
			originTop = node.getOriginTop();
			originBottom = node.getOriginBottom();
			traceID = node.getTraceID();
			morphology = node.getMorphology();
		}
		
		public void setID (int i) {
			nodeID = i;
		}
		
		public int getID() {
			return nodeID;
		}
		
		public void setPhrasalCategory(String s) {
			phrasalCategory = s;
		}
		
		public String getPhrasalCategory() {
			return phrasalCategory;
		}
		
		public void setFunctionalCategory(String s) {
			functionalCategory = s;
		}
		
		public String getFunctionalCategory() {
			return functionalCategory;
		}
		
		public boolean isSubst() {
			return isSubst;
		}
		
		public void setSubst() {
			isSubst = true;
		}
		
		public void setSubst(boolean b) {
			isSubst = b;
		}
		
		public void setLeaf(String l) {
			leaf = l;
			isLeaf = true;
		}
		
		public boolean isLeaf() {
			return isLeaf;
		}
		
		public void setLeaf() {
			isLeaf = true;
		}
		
		public void setNotLeaf() {
			isLeaf = false;
		}
		
		public String getLeaf() {
			return leaf;
		}
		
		public void setLemma(String l) {
			lemma = l;
		}
		
		public String getLemma() {
			return lemma;
		}
		
		public MorphInfo getMorphology() {
			return morphology;
		}
		
		public void setMorphology(MorphInfo m) {
			morphology = m;
		}
	
		public boolean isTrace() {
			return isTrace;
		}
		
		public void setTrace() {
			isTrace = true;
		}
		
		public void setTrace(boolean b) {
			isTrace = b;
		}
		
		public boolean isFiller() {
			return isFiller;
		}
		
		public void setFiller() {
			isFiller = true;
		}
		
		public void setFiller(boolean b) {
			isFiller = b;
		}
		
		public void setTraceID(String s) {
			traceID = s;
		}
		
		public String getTraceID() {
			return traceID;
		}
		
		public void setParent(ElementaryTreeNode p) {
			parent = p;
		}
		
		public ElementaryTreeNode getParent() {
			return parent;
		}
		
		public void addChild(ElementaryTreeNode child) {
			children.add(child);
		}
		
		public void addChild(int i, ElementaryTreeNode child) {
			children.add(i, child);
		}
		
		public void setChild(int i, ElementaryTreeNode child) {
			children.set(i, child);
		}
		
		public int getIndexOfChildWithID(int id) {
			for (int i = 0; i < children.size(); i++) {
				ElementaryTreeNode child = children.get(i);
				if (child.getID() == id) {
					return i;
				}
			}
			return -1;
		}
		
		public int indexOfChild(ElementaryTreeNode child) {
			return children.indexOf(child);
		}
				
		public ArrayList<ElementaryTreeNode> getChildren() {
			return children;
		}
		
		public void removeAllChildren() {
			children = new ArrayList<ElementaryTreeNode>();
		}
		
		public ElementaryTreeNode removeChild(int pos) {
			return children.remove(pos);
		}
		
		public boolean removeChild(ElementaryTreeNode child) {
			return children.remove(child);
		}
		
		public ElementaryTreeNode getHeadChild() {
			return headChild;
		}
		
		public void setHeadChild(ElementaryTreeNode node) {
			headChild = node;
		}
		
		public boolean isHead() {
			return head;
		}
		
		public void setHead() {
			head = true;
		}
		
		public void setHead(boolean b) {
			head = b;
		}
		
		public void setTreeNodeTop(TreeNode node) {
			treeNodeLinkTop = node;
		}
		
		public TreeNode getTreeNodeTop() {
			return treeNodeLinkTop;
		}
		
		public void setTreeNodeBottom(TreeNode node) {
			treeNodeLinkBottom = node;
		}
		
		public TreeNode getTreeNodeBottom() {
			return treeNodeLinkBottom;
		}
		
		public ElementaryTree getOriginBottom() {
			return originBottom;
		}
		
		public void setOriginBottom(ElementaryTree o) {
			originBottom = o;
		}
		
		public ElementaryTree getOriginTop() {
			return originTop;
		}
		
		public void setOriginTop(ElementaryTree o) {
			originTop = o;
		}
		
		public ArrayList<ElementaryTreeNode> getPathUpwards() {
			ArrayList<ElementaryTreeNode> path = new ArrayList<ElementaryTreeNode>();
			ElementaryTreeNode node = this;
			while (node != null) {
				path.add(node);
				node = node.getParent();
			}
			return path;
		}
		
		public String toString() {
			StringBuffer sb = new StringBuffer();
			sb.append(nodeID);
			sb.append(": ");
			sb.append(phrasalCategory);
			sb.append("-");
			sb.append(functionalCategory);
			if (isLeaf()) {
				sb.append(" ");
				sb.append(leaf);
			}
			if (isSubst()) {
				sb.append("!");
			}
			return sb.toString();
		}
}
