package pltag.german;

import java.util.HashMap;
import java.util.regex.Pattern;



public class LeafNode extends TreeNode {
	
	private String leaf;
	private String lemma;
	private boolean isTrace = false;

	public LeafNode(int id, String cat, String l) {
		super(id, cat);
		leaf = l;
	}
	
	public LeafNode(int id, String cat, String l, HashMap<String, Trace> traceList) {
		super(id, cat);
		leaf = l.trim();
		traceList = treatTrace(traceList);
	}
	
	public LeafNode(IDGenerator idgen, String cat, String l) {
		super(idgen, cat);
		leaf = l.trim();
	}
	
	
	protected HashMap<String, Trace> treatTrace(HashMap<String, Trace> traceList) {
		if( this.getFullCategory().startsWith("*T")){ // a hole
			super.traceID = this.getPhrasalCategory();
			traceList = makeTrace(traceList, super.traceID);
			isTrace = true;
			lemma = "--";
		}
		else if (getFunctionalCategory().startsWith("*T") ){ // a filler
			super.traceID = getFunctionalCategory();
			traceList = makeTrace(traceList, traceID);	
		}
		return traceList;
	}

	public boolean isLeaf() {
		return true;
	}
	
	public String getLeafString() {
		return leaf;
	}
	
	public void setMorphology(String morph, Pattern p) {
		super.morphology = new MorphInfo(morph, p);
	}
	
	
	public void setLemma(String l) {
		lemma = l.trim();
	}
	
	public String getLemma() {
		return lemma;
	}
	
	public void setIsTrace() {
		isTrace = true;
	}
	
	public void setIsNoTrace() {
		isTrace = false;
	}
	
	public boolean isTrace() {
		return isTrace;
	}
	
	public String toString(){
		StringBuffer sb = new StringBuffer();
		sb.append(super.getID());
		sb.append(": ");
		sb.append(super.getFullCategory());
		if (super.isFiller()) {
			sb.append("-");
			sb.append(super.traceID);
		}
		sb.append("  ");
		sb.append(leaf);
		if (morphology != null) {
			sb.append("  ");
			sb.append(morphology);
		}
		if (super.isHead()) {
			sb.append("  HEAD");
		}
		return sb.toString();
	}

}
