package pltag.german;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;


public class LexEntry {
	
	private LeafNode leafLink;
	private TreeNode root;
	private HashSet<TreeNode> nodes;
	//private HashSet<LeafNode> otherAnchors;
	
	private boolean adjTree = false;
	private TreeNode foot;
	
	private boolean multiAnchoredTrees;
	private ArrayList<LexEntry> associates = new ArrayList<LexEntry>();
	
	private ElementaryTree eTree;
	
	public LexEntry(TreeNode r, boolean ma) {
		root = r;
		nodes = new HashSet<TreeNode>();
		//otherAnchors = new HashSet<LeafNode>();
		multiAnchoredTrees = ma;
	}
	
	public TreeNode getRoot() {
		return root;
	}
	
	public void setRoot(TreeNode r) {
		root = r;
	}
	
	public LeafNode getLeafLink() {
		return leafLink;
	}
	
	public void setLeafLink(LeafNode l) {
		leafLink = l;
	}
	
	public ArrayList<LexEntry> getAssociates() {
		return associates;
	}
	
	public void addAssociate(LexEntry l) {
		if (!associates.contains(l)) {
			associates.add(l);
		}
	}
	
	
//	public void addAnchor(LeafNode l) {
//		otherAnchors.add(l);
//	}
	
	public void addNode(TreeNode node) {
		nodes.add(node);
	}
	
	public boolean containsNode(TreeNode node) {
		return nodes.contains(node);
	}
	
	public HashSet<TreeNode> getNodes() {
		return nodes;
	}
	
	public boolean isAdjTree() {
		return adjTree;
	}
	
	public void setAdjTree() {
		adjTree = true;
	}
	
	public void setAdjTree(boolean b) {
		adjTree = b;
	}
	
	public TreeNode getFoot() {
		return foot;
	}
	
	public void setFoot(TreeNode node) {
		foot = node;
	}
	
	public void setElementaryTree(ElementaryTree e) {
		eTree = e;
	}
	
	public ElementaryTree getElementaryTree() {
		return eTree;
	}
	
	public void markNodesAsPredicted() {
		for (TreeNode node : nodes) {
			if (node.getBottom() == this) {
				node.setPredictedBottom();
			}
			if (node.getTop() == this) {
				node.setPredictedTop();
			}
		}
	}
	
	public String toString() {
		// word
		StringBuffer bf = new StringBuffer(leafLink.getLeafString());
		bf.append(" ");
		// lemma
		String lemma = leafLink.getLemma();
		if (lemma == null || lemma.equals("") || lemma.equals("--")) {
			lemma = leafLink.getLeafString();
		}
		bf.append(lemma);
		bf.append("\t");
		
		boolean error = false;
		// ADJ, ARG or MOD
		if (this.isAdjTree()) {
			bf.append(NodeStatus.ADJ);
			if (foot == null) {
				error = true;
			}
			if (!root.getPhrasalCategory().equals(foot.getPhrasalCategory())) {
				error = true;
			}
		}
		else {
			bf.append(root.getNodeStatus().toString().replace("MA", ""));
		}
		
		// elementary tree
		bf.append("\t");
		toStringLexTree(root, bf, true);
		
		if (error) {
			System.err.println("### Error in ADJ tree: " + bf.toString().trim());
		}
		
		return bf.toString().trim();
		
//		if (otherAnchors.size() == 0) {
//			return bf.toString().trim();
//		}
//		else {
//			StringBuffer bf2 = new StringBuffer(" ");
//			if (otherAnchors.size() > 0) {
//				// append the other anchors of MAT
//				// make sure they are always in the same (alphabetical) order
//				List<String> otherAnchorStrings = new ArrayList<String>(6);
//				for (LeafNode a : otherAnchors) {
//					otherAnchorStrings.add(a.getLeafString());
//				}
//				Collections.sort(otherAnchorStrings);
//				for (String a : otherAnchorStrings) {	
//					bf2.append(a);
//					bf2.append(" ");
//				}
//			}
//			return bf.toString().trim() + "\t" + bf2.toString().trim();
//		}
	}
	
	private void toStringLexTree(TreeNode node, StringBuffer bf, boolean root) {
		// Remember that node
		TreeNode original = node;
		
		// Skip adjunction trees
		while (node.getBottom() != node.getTop() && node.getBottom().isAdjTree() &&
				!node.getBottom().equals(this) && !node.equals(this.getFoot())) {
			node = node.getBottom().getFoot();
		}
				
		bf.append("(");

		bf.append(node.getPhrasalCategory());
		if (original.isFiller() && original.getTop() != null && original.getTop().equals(this)) { // keep track of moved constituents
			bf.append("-");
			bf.append(original.traceID);
		}
		if (!original.getFunctionalCategory().equals("--")) {
			bf.append("-");
			bf.append(original.getFunctionalCategory());
		}
		
		if (original.getTop() != null && original.getTop().equals(this)) {
			bf.append("^x");
		}
		else {
			bf.append("^null");
		}
		if (node.getBottom() != null && node.getBottom().equals(this)) {
			bf.append("_x");
		}
		else if (multiAnchoredTrees && node.getBottom() != null && 
				original.getNodeStatus() == NodeStatus.MAARG &&
				node.getBottom().equals(original.getMATLink())) {
			int c = node.getBottom().getLeafLink().getID();
			bf.append("_");
			bf.append(c);
			
			bf.append(" ");
			if (node.isLeaf()) {
				bf.append(((LeafNode)node).getLeafString());
				bf.append("^");
				bf.append(c);
				bf.append("_");
				bf.append(c);
			}
			else {
				for (TreeNode child : node.getChildren()) {
					if (node.getBottom().containsNode(child)) {
						node.getBottom().toStringLexTree(child, bf, false, c);
					}
				}
			}
			bf.append(")");
		}
		else { 
			bf.append("_null");
	
			if (node.getBottom() != original.getTop()) {
				if (original.getNodeStatus() == NodeStatus.ARG ||
						(!multiAnchoredTrees && original.getNodeStatus() == NodeStatus.MAARG)) {
					bf.append("!");
				}
				else if (this.isAdjTree() && node == this.getFoot()) {
					bf.append("*");
				}
			}
		}
		
		// sanity check
		if (original.getTop() == null && node.getBottom() == null) {
			System.err.println("### Extraction error " + bf.toString());
		}
		
		if (root && original.getTop() == node.getBottom() &&
				(original.getNodeStatus() == NodeStatus.MOD)) {
			bf.append("*");
		}
		
		bf.append(" ");
		if (node.isLeaf() && node.getBottom().equals(this)) {
			bf.append(((LeafNode)node).getLeafString());
			if (node.getBottom().getLeafLink() == node) { //anchor
				bf.append("<>");
			}
		}
		else {
			for (TreeNode child : node.getChildren()) {
				if (containsNode(child)) {
					toStringLexTree(child, bf, false);
				}
			}
		}
		bf.append(")");
	}

	public void toStringLexTree(TreeNode node, StringBuffer bf, boolean root,
			int i) {
			
			// Remember that node
			TreeNode original = node;
			
			// Skip adjunction trees
			while (node.getBottom() != node.getTop() && node.getBottom().isAdjTree() &&
					!node.getBottom().equals(this)) {
				node = node.getBottom().getFoot();
			}
					
			bf.append("(");

			bf.append(node.getPhrasalCategory());
			if (original.isFiller()) { // keep track of moved constituents
				bf.append("-");
				bf.append(original.traceID);
			}
			if (!original.getFunctionalCategory().equals("--")) {
				bf.append("-");
				bf.append(original.getFunctionalCategory());
			}
			
			if (original.getTop() != null && original.getTop().equals(this)) {
				bf.append("^");
				bf.append(i);
			}
			else {
				bf.append("^null");
			}
			if (node.getBottom() != null && node.getBottom().equals(this)) {
				bf.append("_");
				bf.append(i);
			}
			else if (multiAnchoredTrees && node.getBottom() != null && 
					node.getNodeStatus() == NodeStatus.MAARG &&
					node.getBottom().equals(node.getMATLink())) {
				int c = node.getBottom().getLeafLink().getID();
				bf.append("_");
				bf.append(c);
				
				bf.append(" ");
				if (node.isLeaf()) {
					bf.append(((LeafNode)node).getLeafString());
					bf.append("^");
					bf.append(c);
					bf.append("_");
					bf.append(c);
				}
				else {
					for (TreeNode child : node.getChildren()) {
						if (node.getBottom().containsNode(child)) {
							node.getBottom().toStringLexTree(child, bf, false, c);
						}
					}
				}
				bf.append(")");
			}
			else { 
				bf.append("_null");
		
				if (node.getBottom() != original.getTop()) {
					if (original.getNodeStatus() == NodeStatus.ARG ||
							(!multiAnchoredTrees && original.getNodeStatus() == NodeStatus.MAARG)) {
						bf.append("!");
					}
					else if (this.isAdjTree() && node == this.getFoot()) {
						bf.append("*");
					}
				}
			}
			
			if (root && original.getTop() == node.getBottom() &&
					(original.getNodeStatus() == NodeStatus.MOD)) {
				bf.append("*");
			}
			
			bf.append(" ");
			if (node.isLeaf() && node.getBottom().equals(this)) {
				bf.append(((LeafNode)node).getLeafString());
				bf.append("^");
				bf.append(i);
				bf.append("_");
				bf.append(i);
			}
			else {
				for (TreeNode child : node.getChildren()) {
					if (containsNode(child)) {
						toStringLexTree(child, bf, false);
					}
				}
			}
			bf.append(")");
		}
		

}
