package pltag.german;

import java.util.regex.Pattern;


public class MorphInfo {
	
	public enum Case {
		NOM,
		GEN,
		ACC,
		DAT,
		NONE;
		
		public String toString() {
			if (name().equals("NONE")) {
				return "";
			}
			else {
				return name();
			}
		}
	}
	
	
	private String sourceString = "";
	private Pattern delim;
	
	private Case casus;
	// more here if we consider more morphology
	
	public MorphInfo() {
		casus = Case.NONE;
	}
	
	public MorphInfo(String morphString, Pattern p) {
		sourceString = morphString;
		delim = p;
		parse(morphString);
	}
	
	private void parse(String s) {
		String[] m = delim.split(s);
		for (String morph : m ) {
			if (morph.equals("Nom")) {
				casus = Case.NOM;
			}
			else if (morph.equals("Gen")) {
				casus = Case.GEN;
			}
			else if (morph.equals("Dat")) {
				casus = Case.DAT;
			}
			else if (morph.equals("Acc")) {
				casus = Case.ACC;
			}
			// else: test for other values
		}
		if (casus == null) {
			casus = Case.NONE;
		}
	}
	
	public void setCase(Case c) {
		casus = c;
	}
	
	public Case getCase() {
		return casus;
	}
	
	public void setSourceString(String s) {
		sourceString = s;
	}
	
	public String getSourceString() {
		return sourceString;
	}
	
	public String toString() {
		StringBuffer sb = new StringBuffer();
	
		if (casus != null) {
			sb.append(casus.toString().toLowerCase());
		}
		// more here if we consider more morphology
		
		if (sb.length() == 0) {
			return "";
		}
		return "[" + sb.toString() + "]";
	}

	public MorphInfo unify(MorphInfo other) {
		MorphInfo newMorph = new MorphInfo();
		if (casus == Case.NONE) {
			newMorph.setCase(other.getCase());
		}
		else if (other.getCase() == Case.NONE) {
			newMorph.setCase(casus);
		}
		else if (casus == other.getCase()){
			newMorph.setCase(casus);
		}
		else { // clash
			return null;
		}
		return newMorph;
	}
	
	public boolean equals(Object anObject){
		  if ( this == anObject ) return true;
		  if ( !(anObject instanceof MorphInfo) ) return false;
		  MorphInfo other = (MorphInfo)anObject;
		  return
		    this.casus == other.getCase();
		}
}
