package pltag.german;

import java.util.ArrayList;


public class SiblingStatus {
	
	public void determineStatusOfChildren(TreeNode node) {
		String motherCat = node.getPhrasalCategory();

		if (Utils.isCoord(node)) {
			coordDetermineStatus(node);
		}
		else if (motherCat.matches("VNODE")) {
			vnodeDetermineStatus(node);
		}	
		else if (motherCat.matches("AA")) {
			aaDetermineStatus(node);
		}	
		else if (motherCat.matches("AP")) {
			apDetermineStatus(node);
		}
		else if (motherCat.matches("AC")) {
			acDetermineStatus(node);
		}
		else if (motherCat.matches("AVP")) {
			avpDetermineStatus(node);
		}
		else if (motherCat.matches("CH")) {
			chDetermineStatus(node);
		}
		else if (motherCat.matches("DL")) {
			dlDetermineStatus(node);
		}
		else if (motherCat.matches("DP")) {
			dpDetermineStatus(node);
		}
		else if (motherCat.matches("ISU")) {
			isuDetermineStatus(node);
		}
		else if (motherCat.matches("MTA")) {
			mtaDetermineStatus(node);
		}
		else if (motherCat.matches("NM")) {
			nmDetermineStatus(node);
		}
		else if (motherCat.matches("NP")) {
			npDetermineStatus(node);
		}
		else if (motherCat.matches("PCC")) {
			pccDetermineStatus(node);
		}
		else if (motherCat.matches("PN")) {
			pnDetermineStatus(node);
		}
		else if (motherCat.matches("PP")) {
			ppDetermineStatus(node);
		}	
		else if (motherCat.matches("VP|S")) {
			sDetermineStatus(node);
		}
		else if (motherCat.matches("VZ")) {
			vzDetermineStatus(node);
		}
		
	}
	

	private void vnodeDetermineStatus(TreeNode node) {
		NodeStatus ns;
		for (TreeNode child : node.getChildren()) {
			if (!child.isHead()) {
				ns = NodeStatus.MOD;
				child.setNodeStatus(ns);
			}
		}	
	}
	
	private void aaDetermineStatus(TreeNode node) {
		NodeStatus ns;
		for (TreeNode child : node.getChildren()) {
			if (!child.isHead()) {
				if (child.getFunctionalCategory().matches("PM")) {
					ns = NodeStatus.ARG;
				}
				else {
					ns = NodeStatus.MOD;
				}
				child.setNodeStatus(ns);
			}
		}	
	}
	
	private void acDetermineStatus(TreeNode node) {
		NodeStatus ns;
		for (TreeNode child : node.getChildren()) {
			if (!child.isHead() && child.getNodeStatus() == NodeStatus.UNDEF) {
				ns = NodeStatus.MOD;
				child.setNodeStatus(ns);
			}
		}	
	}
	

	private void apDetermineStatus(TreeNode node) {
		NodeStatus ns;
		for (TreeNode child : node.getChildren()) {
			if (!child.isHead()) {
				if (child.getFunctionalCategory().matches("OA|DA|OG|AMS")) {
					ns = NodeStatus.ARG;
				}
				else {
					ns = NodeStatus.MOD;
				}
				child.setNodeStatus(ns);
			}
		}	
	}
	
	private void avpDetermineStatus(TreeNode node) {
		NodeStatus ns;
		for (TreeNode child : node.getChildren()) {
			if (!child.isHead() && child.getNodeStatus() == NodeStatus.UNDEF) {
				if (child.getFunctionalCategory().equals("AVC")) {
					ns = NodeStatus.MAARG;
				}
				else {
					ns = NodeStatus.MOD;
				}
				child.setNodeStatus(ns);
			}
		}	
	}
	
	private void chDetermineStatus(TreeNode node) {
		NodeStatus ns;
		for (TreeNode child : node.getChildren()) {
			if (!child.isHead() && child.getNodeStatus() == NodeStatus.UNDEF) {
				if (child.getFunctionalCategory().equals("UC")) {
					ns = NodeStatus.ARG;
				}
				else {
					ns = NodeStatus.MOD;
				}
				child.setNodeStatus(ns);
			}
		}	
	}
	
	private void dlDetermineStatus(TreeNode node) {
		NodeStatus ns;
		for (TreeNode child : node.getChildren()) {
			if (!child.isHead() && child.getNodeStatus() == NodeStatus.UNDEF) {
				if (child.getFunctionalCategory().matches("DH|RS")) {
					ns =NodeStatus.ARG;
				}
				else {
					ns = NodeStatus.MOD;
				}
				child.setNodeStatus(ns);
			}
		}	
	}
	
	private void dpDetermineStatus(TreeNode node) {
		NodeStatus ns;
		for (TreeNode child : node.getChildren()) {
			if (!child.isHead() && child.getNodeStatus() == NodeStatus.UNDEF) {
				ns = NodeStatus.MOD;
				child.setNodeStatus(ns);
			}
		}	
	}
	
	private void isuDetermineStatus(TreeNode node) {
		NodeStatus ns;
		for (TreeNode child : node.getChildren()) {
			if (!child.isHead() && child.getNodeStatus() == NodeStatus.UNDEF) {
				if (child.getFunctionalCategory().equals("UC")) {
					ns = NodeStatus.MAARG;
				}
				else {
					ns = NodeStatus.MOD;
				}
				child.setNodeStatus(ns);
			}
		}	
	}
	
	private void mtaDetermineStatus(TreeNode node) {
		NodeStatus ns;
		for (TreeNode child : node.getChildren()) {
			if (!child.isHead() && child.getNodeStatus() == NodeStatus.UNDEF) {
				if (child.getFunctionalCategory().equals("ADC")) {
					ns = NodeStatus.MAARG;
				}
				else {
					ns = NodeStatus.MOD;
				}
				child.setNodeStatus(ns);
			}
		}	
	}
	
	private void nmDetermineStatus(TreeNode node) {
		NodeStatus ns;
		boolean nnHead = false;
		if (node.getHeadChild().getPhrasalCategory().matches("NN")) {
			nnHead = true;
		}
		for (TreeNode child : node.getChildren()) {
			if (!child.isHead() && child.getNodeStatus() == NodeStatus.UNDEF) {
				if (child.getFunctionalCategory().equals("NMC")) {
					if (nnHead) {
						ns = NodeStatus.ARG; // 3 Milliarden, eine Million
					}
					else {
						ns = NodeStatus.MOD; /// 600 000
					}
				}
				else {
					ns = NodeStatus.MOD;
				}
				child.setNodeStatus(ns);
			}
		}	
	}
	
	private void npDetermineStatus(TreeNode node) {
		ArrayList<TreeNode> children = node.getChildren();
		// First have a look at the DPs
		ArrayList<Integer> dps = new ArrayList<Integer>(6);
		int ap = -1;
		TreeNode child;
		for (int i = 0; i < children.size(); i++) {
			child = children.get(i);
			if (!child.isHead()) {
				if (child.getPhrasalCategory().matches("DP")) {
					dps.add(i);
				}
				if (child.getFullCategory().matches("AP-NK")) {
					ap = i;
				}
			}
		}	
		// substitution DPs only in front of adjectival material
		if (ap > -1) {
			int i;
			for (i = dps.size()-1; i > -1; i--) {
				if (dps.get(i) > ap) {
					dps.remove(i);
				}
				else {
					break;
				}
			}
		}
		if (dps.size() == 1) { // one DP
			children.get(dps.get(0)).setNodeStatus(NodeStatus.ARG);
		}
		else if (dps.size() >  1 && dps.get(0)+1 == dps.get(1)) { // two successive DPs (or more)
			int dp1 = dps.get(0);
			int dp2 = dps.get(1);
			// check whether one of them is a PIAT
			TreeNode child1 = children.get(dp1);
			while (!child1.isLeaf()) {
				child1 = child1.getHeadChild();
			}
			TreeNode child2 = children.get(dp2);
			while (!child2.isLeaf()) {
				child2 = child2.getHeadChild();
			}
			if (child1.getPhrasalCategory().equals("PIAT")) {
				children.get(dp1).setNodeStatus(NodeStatus.MOD);
				children.get(dp2).setNodeStatus(NodeStatus.ARG);
			}
			else if (child2.getPhrasalCategory().equals("PIAT")) {
				children.get(dp2).setNodeStatus(NodeStatus.MOD);
				children.get(dp1).setNodeStatus(NodeStatus.ARG);
			}
			else { // no PIAT: we assume the left-most DP marks the substitution site
				children.get(dp2).setNodeStatus(NodeStatus.MOD);
				children.get(dp1).setNodeStatus(NodeStatus.ARG);
			}
		}
		
		// now look at the rest
		// TODO refine, OC, OP OG should probably be substituted
		for (TreeNode child2 : children) {
			if (child2.getNodeStatus() == NodeStatus.UNDEF && !child2.isHead()) {
				child2.setNodeStatus(NodeStatus.MOD);
			}
		}
	}
	
	private void pccDetermineStatus(TreeNode node) {
		NodeStatus ns;
		for (TreeNode child : node.getChildren()) {
			if (!child.isHead() && child.getNodeStatus() == NodeStatus.UNDEF) {
				if (child.getFunctionalCategory().matches("COB")) {
					ns = NodeStatus.ARG;
				}
				else {
					ns = NodeStatus.MOD;
				}
				child.setNodeStatus(ns);
			}
		}	
	}
	
	private void pnDetermineStatus(TreeNode node) {
		NodeStatus ns;
		for (TreeNode child : node.getChildren()) {
			if (!child.isHead() && child.getNodeStatus() == NodeStatus.UNDEF) {
				if (child.getFunctionalCategory().matches("PNC")) {
					ns = NodeStatus.ARG;
				}
				else {
					ns = NodeStatus.MOD;
				}
				child.setNodeStatus(ns);
			}
		}	
	}
	
	private void ppDetermineStatus(TreeNode node) {
		NodeStatus ns;
		for (TreeNode child : node.getChildren()) {
			if (!child.isHead() && child.getNodeStatus() == NodeStatus.UNDEF) {
				if (child.getFunctionalCategory().matches("PC")) {
					ns = NodeStatus.ARG;
				}
				else if (child.getFunctionalCategory().matches("AC")) { // circumpositions -> MAT
					ns = NodeStatus.MAARG;
				}
				else {
					ns = NodeStatus.MOD;
				}
				child.setNodeStatus(ns);
			}
		}	
	}
	
	private void sDetermineStatus(TreeNode node) {
		NodeStatus ns;
		for (TreeNode child : node.getChildren()) {
			if (!child.isHead()) {
				if (child.getFunctionalCategory().matches(Utils.verbalArgumentFunctionalCats)) {
					ns = NodeStatus.ARG;
				}
				else if (child.getFunctionalCategory().matches("SVP")) {
					ns = NodeStatus.MAARG;
				}
				else if (child.getFunctionalCategory().matches("CVC")) {
					ns = NodeStatus.MAARG;
					// make sure that all parts of the collocation (also the NP) are included
					if (child.getPhrasalCategory().equals("PP")) {
						for (TreeNode grandChild : child.getChildren()) {
							if (grandChild.getFunctionalCategory().equals("PC")) {
								// the prep complement sometimes is a coordination 
								// (e.g. erzählen von Diffamierung und Demontage);
								// TODO
							//	if (!grandChild.getPhrasalCategory().equals("CNP") &&
							//			!grandChild.isCoordinatedCat()) {
									grandChild.setNodeStatus(NodeStatus.MAARG);
							//	}
							}
						}
					}
				}
//				else if (node.getPhrasalCategory().equals("VP") && child.getFunctionalCategory().matches("HD") &&
//						(child.getPhrasalCategory().matches("VM.*?|VA.*?") ||
//								(child.getPhrasalCategory().matches("VZ") && 
//								child.getHeadChild().getPhrasalCategory().matches("VM.*?|VA.*?")))) {
//					ns = NodeStatus.ADJ;
//				}
				// in first if statement now
//				else if (child.getFunctionalCategory().matches("OC")) {
//					ns = NodeStatus.ARG;
//				}
				else {
					ns = NodeStatus.MOD;
				}
				child.setNodeStatus(ns);
			}
		}	
	}

	private void vzDetermineStatus(TreeNode node) {
		NodeStatus ns;
		for (TreeNode child : node.getChildren()) {
			if (!child.isHead()) {
				if (child.getFunctionalCategory().matches("PM")) {
					ns = NodeStatus.ARG;
				}
				else {
					ns = NodeStatus.MOD;
				}
				child.setNodeStatus(ns);
			}
		}	
	}
	
	private void coordDetermineStatus(TreeNode node) {
		NodeStatus ns;
		for (TreeNode child : node.getChildren()) {
			if (!child.isHead()) {
				if (child.isLeaf() && ((LeafNode)child).isTrace()) {
					ns = NodeStatus.MOD;
				}
				else if (child.getFunctionalCategory().matches("CD|CJ")) {
					ns = NodeStatus.ARG;
				}
				else {
					ns = NodeStatus.MOD;
				}
				child.setNodeStatus(ns);
			}
		}	
		
	}

}
