package pltag.german;

import java.util.ArrayList;
import java.util.HashMap;





public class TreeNode {
	
	private TreeNode parent;
	private boolean hasParent = false;
	private TreeNode rightSibling, leftSibling;
	private ArrayList<TreeNode> children= new ArrayList<TreeNode>();
	private TreeNode headChild;
	private boolean head = false;
	
	protected MorphInfo morphology = new MorphInfo();
	
	private String fullCategory;
	private String phrasalCategory;
	private String functionalCategory;
	private boolean coordinatedCat = false;
	private boolean unlikeCoordinationCat = false;
	
	private ArrayList<SecondaryEdge> secondaryEdges = new ArrayList<SecondaryEdge>();
	
	private int nodeID = -999; // undef
	
	protected String traceID;
	protected boolean hasTrace = false;
	protected Trace trace;
	protected boolean isFiller = false;
	
	private LexEntry lexTop;
	private LexEntry lexBottom;
	private NodeStatus nodeStatus = NodeStatus.UNDEF;
	private NodeStatus nodeStatusBottom = NodeStatus.UNDEF;
	private boolean isFoot = false;
	private LexEntry matLink;
	
	private boolean predictedTop = false;
	private boolean predictedBottom = false;
	
	private boolean forBinarization = false;
	private boolean hadTraceChild = false;
	
	public TreeNode(int id, String cat) {
		nodeID = id;
		fullCategory = cat.trim();
		if (cat.contains("-")) {
			int i = cat.indexOf("-");
			phrasalCategory = cat.substring(0, i).trim();
			functionalCategory = cat.substring(i+1).trim();
		}
		else {
			phrasalCategory = cat.trim();
			functionalCategory = "--";
		}
	}
	
	public TreeNode(IDGenerator idgen, String cat) {
		this(idgen.getNewID(), cat);
	}
	
	public TreeNode(IDGenerator idgen, String cat, boolean forBin) {
		this(idgen, cat);
		setInsertedForBinarization(forBin);
	}
	
	public boolean isLeaf() {
		return false;
	}
	
	public int getID() {
		return nodeID;
	}
	
	public void setID(int id) {
		nodeID = id;
	}
	
	public String getFullCategory() {
		return fullCategory;
	}

	public String getPhrasalCategory() {
		return phrasalCategory;
	}
	
	public String getFunctionalCategory() {
		return functionalCategory;
	}
	
	public void setPhrasalCategory(String cat) {
		phrasalCategory = cat;
		// also fix full category string
		int i = fullCategory.indexOf("-");
		if (i > -1) {
			fullCategory = cat + fullCategory.substring(i);
		}
		else {
			fullCategory = cat;
		}
	}
	
	public void setFunctionalCategory(String c) {
		c = c.trim();
		functionalCategory = c;
		// also fix full category string
		int i = fullCategory.indexOf("-");
		if (i > -1) {
			fullCategory = fullCategory.substring(0, i+1) + c;
		}
		else {
			fullCategory += "-" + c;
		}
	}
	
	public boolean isFiller() {
		return isFiller;
	}
	
	public void setFiller(boolean b) {
		isFiller = b;
	}
	
	public boolean isCoordinatedCat() {
		return coordinatedCat;
	}
	
	public void setCoordinatedCat(boolean b) {
		coordinatedCat = b;
	}
	
	public void setCoordinatedCat() {
		setCoordinatedCat(true);
	}
	
	public boolean isUnlikeCoordination() {
		return unlikeCoordinationCat;
	}
	
	public void setUnlikeCoordination(boolean b) {
		unlikeCoordinationCat = b;
	}
	
	public void setParent(TreeNode node) {
		parent = node;
		hasParent = true;
	}
	
	public TreeNode getParent() {
		return parent;
	}
	
	public boolean hasParent() {
		return hasParent;
	}

	public void addChild(TreeNode child) {
		children.add(child);		
	}
	
	public void addChild(int pos, TreeNode child) {
		children.add(pos, child);
	}
	
	public TreeNode removeChild(int pos) {
		return children.remove(pos);
	}
	
	public boolean removeChild(TreeNode child) {
		return children.remove(child);
	}
	
	public int indexOfChild(TreeNode child) {
		return children.indexOf(child);
	}
	
	public ArrayList<TreeNode> getChildren() {
		return children;
	}
	
	public TreeNode getFirstChild() {
		return children.get(0);
	}
	
	public TreeNode getLastChild() {
		return children.get(children.size()-1);
	}

	public boolean isLeftMostChild(TreeNode child) {
		return (children.get(0) == child);
	}
	
	public boolean isRightMostChild(TreeNode child) {
		return (children.get(children.size()-1) == child);
	}
	
	public void setLeftSibling(TreeNode node) {
		leftSibling = node;
	}

	public void setRightSibling(TreeNode node) {
		rightSibling = node;
	}
	
	public TreeNode getLeftSibling() {
		return leftSibling;
	}
	
	public TreeNode getRightSibling() {
		return rightSibling;
	}
	
	public void setHeadChild(TreeNode node) {
		headChild = node;
	}
	
	public TreeNode getHeadChild() {
		return headChild;
	}
	
	public void setHead() {
		head = true;
	}
	
	public void setNotHead() {
		head = false;
	}
	
	public boolean isHead() {
		return head;
	}
	
	public MorphInfo getMorphology() {
		return morphology;
	}
	
	public void addSecondaryEdge(SecondaryEdge sec) {
		secondaryEdges.add(sec);
	}
	
	public ArrayList<SecondaryEdge> getOutgoingSecondaryEdges() {
		ArrayList<SecondaryEdge> outgoing = new ArrayList<SecondaryEdge>();
		for (SecondaryEdge sec : secondaryEdges) {
			if (sec.getStartNode() == this) {
				outgoing.add(sec);
			}
		}
		return outgoing;
	}
	
	public ArrayList<SecondaryEdge> getIncomingSecondaryEdges() {
		ArrayList<SecondaryEdge> incoming = new ArrayList<SecondaryEdge>();
		for (SecondaryEdge sec : secondaryEdges) {
			if (sec.getEndNode() == this) {
				incoming.add(sec);
			}
		}
		return incoming;
	}
	
	public ArrayList<SecondaryEdge> getSecondaryEdges() {
		return secondaryEdges;
	}
	
	public LexEntry getTop() {
		return lexTop;
	}
	
	public void setTop(LexEntry l) {
		lexTop = l;
	}
	
	public LexEntry getBottom() {
		return lexBottom;
	}
	
	public void setBottom(LexEntry l) {
		lexBottom = l;
	}
	
	public NodeStatus getNodeStatus() {
		return nodeStatus;
	}
	
	public void setNodeStatus(NodeStatus ns) {
		nodeStatus = ns;
	}
	
	public NodeStatus getNodeStatusBottom() {
		return nodeStatusBottom;
	}
	
	public void setNodeStatusBottom(NodeStatus ns) {
		nodeStatusBottom = ns;
	}
	
	public boolean isFoot() {
		return isFoot;
	}
	
	public void setIsFoot() {
		isFoot = true;
	}
	
	public void setIsFoot(boolean f) {
		isFoot = f;
	}
	
	public LexEntry getMATLink() {
		return matLink;
	}
	
	public void setMATLink(LexEntry l) {
		matLink = l;
	}
	
	public boolean isPredictedTop() {
		return predictedTop;
	}
	
	public void setPredictedTop() {
		predictedTop = true;
	}
	
	public void setPredictedTop(boolean b) {
		predictedTop = b;
	}
	
	public boolean isPredictedBottom() {
		return predictedBottom;
	}
	
	public void setPredictedBottom() {
		predictedBottom = true;
	}
	
	public void setPredictedBottom(boolean b) {
		predictedBottom = b;
	}
	
	public boolean insertedForBinarization() {
		return forBinarization;
	}
	
	public void setInsertedForBinarization(boolean b) {
		forBinarization = b;
	}
	
	public boolean hadTraceChild() {
		return hadTraceChild;
	}
	
	public void setHadTraceChild(boolean b) {
		hadTraceChild = b;
	}

	public boolean subtreeContains(TreeNode n) {
		if (this == n) {
			return true;
		}
		boolean found = false;
		for (TreeNode child : getChildren()) {
			found = child.subtreeContains(n);
			if (found) break;
		}
		return found;
	}
	
	protected HashMap<String, Trace> makeTrace(HashMap<String,Trace> traceList, String traceID) {
		Trace trace;
		if (! traceList.containsKey(traceID)){ // this is the "first part" of the trace (i.e. hole or filler)
			trace = new Trace(this);			
		}
		else { // this is the "second part" of the trace
			trace = traceList.get(traceID);
			trace.addInfo(this);
			// fix the nodeIDs
			if (trace.getHole().getID() < 0 &&
					trace.getFiller().getID() > 1000) {
				int holeID = trace.getHole().getID();
				trace.getFiller().setID((-1)*holeID);
			}
			//if (trace.getHole().getID() == -999) {
				//trace.getHole().setID(-trace.getFiller().getID());
			//}
			//else if (trace.getFiller().getID() == -999) {
				//int holeID = trace.getHole().getID();
				//trace.getFiller().setID((-1)*holeID);
			//}
			
			// fix the functional category of the filler
			trace.getFiller().setFiller(true);
			trace.getFiller().setFunctionalCategory(trace.getHole().getFunctionalCategory());
		}
		traceList.put(traceID, trace);
		hasTrace = true;
		this.trace = trace;
		return traceList;
	}
	
}
