package pltag.german;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.regex.Matcher;
import java.util.regex.Pattern;


public class Utils {
	
	public static String coordinatedCats = "CAC|CAP|CAVP|CCP|CNP|CO|CPP|CS|CVP|CVZ";
	public static String modifierFunctionalCats = "MO|RE|RC|MNR|CC|APP|AG|PAR|PG";
	public static String verbalArgumentFunctionalCats = "OC|SB|OA|OA2|OG|OP|PD|DA|SBP|EP";
	
	private static Pattern pCoordinatedCats = Pattern.compile(coordinatedCats);
	private static Pattern pPunct = Pattern.compile("\\$,|\\$\\.|\\$\\*LRB\\*");

	public static String subtreeToString(TreeNode node) {
		StringBuffer s = new StringBuffer();
		toString(s, node, 0);
		return s.toString();		
	}

	private static void toString(StringBuffer s, TreeNode node, int level) {
		for (int i = 0; i < level; i++) {
			s.append("  ");
		}
		s.append(node.toString() + "\n");
		for (TreeNode child : node.getChildren()) {
			toString(s, child, level+1);
		}
	}
	
	public static void printMemory(String s) {
		Runtime rt = Runtime.getRuntime();
		long m = rt.totalMemory() - rt.freeMemory();
		System.out.println("$$$" + s + ": " + m/1000 + " KB");
	}
	
	public static boolean isPunct(String cat) {
		Matcher m = pPunct.matcher(cat);
		return m.find();
	}
	
	public static boolean isCoord(TreeNode node) {
		if (node.isCoordinatedCat()) {
			return true;
		}
		Matcher m = pCoordinatedCats.matcher(node.getPhrasalCategory());
		return m.find();
	}
	
	public static boolean isAuxiliaryOrModal(TreeNode node) {
		if (node.getPhrasalCategory().matches("(VA|VM).*")) {
			return true;
		}
//		if (node.getPhrasalCategory().matches("VZ")) {
//			for (TreeNode child : node.getChildren()) {
//				if (child.getFunctionalCategory().matches("HD")
//						&& child.getPhrasalCategory().matches("(VA|VM).*")) {
//					return true;
//				}
//			}
//		}
		return false;
	}

	/*
	 * Check whether child is (or leads to) the rightmost leaf of the Tiger tree
	 */
	public static boolean isRightPunctPeriphery(TigerTree tigerTree, TreeNode child) {
		while (!child.isLeaf()) {
			child = child.getHeadChild();
		}
		if (!Utils.isPunct(child.getPhrasalCategory())) {
			return false;
		}
		
		ArrayList<LeafNode> leaves = tigerTree.getLeafNodes();
		int i = leaves.indexOf(child);
		if (i == leaves.size()-1) {
			return true;
		}
		for (int j = i+1; j < leaves.size(); j++) {
			if (!Utils.isPunct(leaves.get(j).getPhrasalCategory())) {
				return false;
			}
		}
		return true;
	}

	public static boolean isWellFormed(ElementaryTree eTree,
			StringBuffer errorBuffer, HashMap<String, Trace> traceListTigerTree) {
		if (eTree.getType() == NodeStatus.ADJ) {
			ElementaryTreeNode root = eTree.getRoot();
			ElementaryTreeNode foot = eTree.getFoot();
			if (!root.getPhrasalCategory().equals(
								foot.getPhrasalCategory())) {
				errorBuffer.append("### Categories don't match. " + eTree.toStringStruct(traceListTigerTree));
				return false;
			}
			if (root.getMorphology().getCase() != foot.getMorphology().getCase()) {
				errorBuffer.append("### Cases don't match. " + eTree.toStringStruct(traceListTigerTree));
				return false;
			}
		}
		// TODO more
		return true;
	}

}
