package pltag.german.analysis;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class TigerDataDivider {

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		String tigerPath = args[0];
		TigerDataDivider.divide(tigerPath);

	}
	
	private static void divide(String tigerPath) {
		File tigerFile = new File(tigerPath);
		File directory = tigerFile.getParentFile();
		File trainFile = new File(directory + File.separator + tigerFile.getName() + ".train");
		File devFile = new File(directory + File.separator + tigerFile.getName() + ".dev");
		File testFile = new File(directory + File.separator + tigerFile.getName() + ".test");

		ArrayList<String> lines = new ArrayList<String>();
		
		FileReader fr ;
		BufferedReader input = null;
		BufferedWriter train = null;
		BufferedWriter dev = null;
		BufferedWriter test = null;
		
		try {			
			fr = new FileReader(tigerFile);
			input = new BufferedReader(fr);
			train = new BufferedWriter(new FileWriter(trainFile));
			dev = new BufferedWriter(new FileWriter(devFile));
			test = new BufferedWriter(new FileWriter(testFile));
			
			String line = null;
			Pattern p = Pattern.compile("%% Sent ([0-9]+)");
			String sentID = "";
			
			int s = 0;
			int sTrain = 0;
			int sDev = 0;
			int sTest = 0;
			/*
			 * readLine is a bit quirky :
			 * it returns the content of a line MINUS the newline.
			 * it returns null only for the END of the stream.
			 * it returns an empty String if two newlines appear in a row.
			 */	
			while (( line = input.readLine()) != null){
				Matcher m = p.matcher(line);
				if (m.find()) {
					
					// a new sentence starts, treat the previous one
					if (!sentID.equals("")) {
						int id = Integer.parseInt(sentID);
						int mod = id % 20;
						if (mod == 18) { // dev
							for (String l : lines) {
								dev.write(l + "\n");
							}
							sDev++;
						}
						else if (mod == 19) { // test
							for (String l : lines) {
								test.write(l + "\n");
							}
							sTest++;
						}
						else { // train
							for (String l : lines) {
								train.write(l + "\n");
							}
							sTrain++;
						}
					}
					
					lines = new ArrayList<String>();
					sentID = m.group(1);
					s++;
				}
				lines.add(line);
			}	
			
			// for the last sentence
			int id = Integer.parseInt(sentID);
			int mod = id % 20;
			if (mod == 18) { // dev
				for (String l : lines) {
					dev.write(l + "\n");
				}
				sDev++;
			}
			else if (mod == 19) { // test
				for (String l : lines) {
					test.write(l + "\n");
				}
				sTest++;
			}
			else { // train
				for (String l : lines) {
					train.write(l + "\n");
				}
				sTrain++;
			}
			
			System.out.println(s);
			System.out.println("Train " + sTrain);
			System.out.println("Dev " + sDev);
			System.out.println("Test " + sTest);
			
		}
		catch (FileNotFoundException e) {
			System.err.println(e.getMessage());
			//e.printStackTrace();
		}
		catch (IOException ex){
			ex.printStackTrace();
		}
		finally {
			try {
				if (input!= null) {
					input.close();
				}
				train.close();
				test.close();
				dev.close();
			}
			catch (IOException ex) {
				ex.printStackTrace();
			}
		}
	}

}
